#include "config.h"
#include "lpc23xx.h"
#include "init.h"

static void feed(void){
  PLLFEED=0xAA;
  PLLFEED=0x55;
}

#define PLOCK (1<<26)

void MCUInit(void){
	volatile int i;
	
	SCS |= (1<<5);					// main Oocillator enable
	while( (SCS&(1<<6)) == 0 ) ;	// Wait for main osc stabilize
	CLKSRCSEL = 0x01;				// Use main oscillator

	// Setting Multiplier and Divider values
	// Main osc(Fin) = 12 MHz
	//
	// if Fcco = 288 MHz,
	// M = (Fcco x N) / (2 x Fin) = (288 x 1) / (2 x 12) = 12 (N=1).
	//
	// In this case, M=12 and N=1.
	// 	
  	PLLCFG = ((CLK_PLL_M-1) | ((CLK_PLL_N-1)<<16));
  	feed();
  
	// Enabling the PLL
	PLLCON=0x1;
	feed();
  
	// Setup CPU clock divider (cclk = 288/4 = 72MHz)
	CCLKCFG = (CLK_CCLK_DIV-1);		// This value must be an odd value (see Datasheet)
	
	// Setup USB clock divider (usbclk = 288/6 = 48MHz)
	USBCLKCFG = (CLK_USBCLK_DIV-1);
	
	// Setup Peripheral Clock
	//PCLKSEL0
	//PCLKSEL1

	
	
	// Wait for the PLL to lock to set frequency
	while(!(PLLSTAT & PLOCK)) ;
    	
	// Check the PLL divider 
	while( (PLLSTAT&0xffffff) != ((CLK_PLL_M-1) | ((CLK_PLL_N-1)<<16)) ) ;

	
	// Connect the PLL as the clock source
	PLLCON=0x3;
	feed();
	// Check connect bit(PLLC) status
	while ( ((PLLSTAT & (1 << 25)) == 0) );
	
	
	for(i=0; i<2000; i++) ;

	// Enabling MAM
	//MAMCR = 0x0;	// stop MAM
	// 4 clks (over 60MHz), 3 clk (40 MHz to 60 MHz)
	MAMTIM = (CLK_CCLK >= 60000000)? 0x4 : 0x3;
	MAMCR  = 0x01;	// MAM functions partially enabled 
	MEMMAP = 0x01;		// changing to User Flash Mode
}
